@@ -48,7 +48,7 @@ expandCmd <- function(cmd, alphaStart=TRUE) {
   #hyphens <- gregexpr("(?!<(\\*|\\.))\\w+(?!(\\*|\\.))\\s*-\\s*(?!<(\\*|\\.))\\w+(?!(\\*|\\.))", cmd, perl=TRUE)[[1]]
 
   #support trailing @XXX. Still still fail on Trait1-Trait3*XXX
-  hyphens <- gregexpr("(?!<(\\*|\\.))\\w+(?!(\\*|\\.))\\s*-\\s*(?!<(\\*|\\.))\\w+(?!(\\*|\\.))(@[\\d\\.-]+)?", cmd, perl=TRUE)[[1]]
+  hyphens <- gregexpr("(?!<(\\*|\\.))\\w+(?!(\\*|\\.))\\s*-\\s*(?!<(\\*|\\.))\\w+(?!(\\*|\\.))(@[\\d\\.\\-]+)?", cmd, perl=TRUE)[[1]]
 
   #Promising, but this is still failing in the case of x3*1 -4.25*x4
   #On either side of a hyphen, require alpha character followed by alphanumeric
@@ -138,9 +138,10 @@ parseFixStart <- function(cmd) {
 
   #support ESEM-like syntax: F BY a1* a2*
   #The easy path: putting in 1s before we proceed on parsing
-  cmd <- gsub("([A-z]+\\w*)\\s*\\*(?=\\s+[A-z]+|\\s*$)", "\\1*1", cmd, perl=TRUE)
+  # Mar2023 bugfix: support parenthesis after * in case a parameter constraint comes next
+  cmd <- gsub("([A-z]+\\w*)\\s*\\*(?=\\s+\\(?[A-z]+|\\s*$)", "\\1*1", cmd, perl=TRUE)
 
-  if ((fixed.starts <- gregexpr("[\\w\\.-]+\\s*([@*])\\s*[\\w\\.-]+", cmd, perl=TRUE)[[1]])[1L] > 0) { #shouldn't it be \\*, not * ?! Come back to this.
+  if ((fixed.starts <- gregexpr("[\\w\\.\\-$]+\\s*([@*])\\s*[\\w\\.\\-]+", cmd, perl=TRUE)[[1]])[1L] > 0) { #shouldn't it be \\*, not * ?! Come back to this.
     for (f in 1:length(fixed.starts)) {
 
       #capture above obtains the fixed/start character (@ or *), whereas match obtains the full regex match
@@ -697,6 +698,36 @@ mplus2lavaan.modelSyntax <- function(syntax) {
     }
   }
 
+  # new threshold syntax shifts things to the form:
+  # VAR | t1 + t2 + t3 (left to write ordering)
+  # Parameter labels, fixed values, and starting values are tacked on in the usual way, like
+  # VAR | 5*t1 + start(1.5)*t2 + par_label*t3 (left to write ordering)
+
+  thresh_lines <- grep("^\\s*[A-z]+\\w*\\|\\d+", lavaan.out, perl=TRUE)
+  if (length(thresh_lines) > 0L) {
+    thresh_vars <- unname(sub("^\\s*([A-z]+\\w*).*", "\\1", lavaan.out[thresh_lines], perl=TRUE))
+    thresh_split <- split(thresh_lines, thresh_vars)
+    drop_elements <- c()
+    for (i in seq_along(thresh_split)) {
+      this_set <- lavaan.out[thresh_split[[i]]]
+      tnum <- as.integer(sub("^\\s*[A-z]+\\w*\\|(\\d+)\\s*.*", "\\1", this_set))
+      this_set <- this_set[order(tnum)] # ensure that threshold numbering matches ascending order
+      this_set <- sub("[^~]+\\s*~\\s*", "", this_set, perl=T) # drop variable and ~
+
+      # convert to new t1, t2 syntax by combining modifiers with threshold numbers
+      this_set <- sapply(seq_along(this_set), function(j) {
+        #gsub("[^~]+\\s*~\\s*([\\w\\.\\-]+\\*)*1", paste0("\\1t", j), this_set[j], perl=TRUE)
+        gsub("([\\w\\.\\-]+\\*)*1", paste0("\\1t", j), this_set[j], perl=TRUE)
+      })
+
+      new_str <- paste(names(thresh_split)[i], "|", paste(this_set, collapse=" + "))
+      # replace in model string on the first line having relevant syntax
+      lavaan.out[thresh_split[[i]][1]] <- new_str
+      drop_elements <- c(drop_elements, thresh_split[[i]][-1])
+    }
+    lavaan.out <- lavaan.out[-drop_elements]
+  }
+
 
   #tack on constraint syntax, if included
   lavaan.out <- c(lavaan.out, con_syntax)
