@@ -188,6 +188,9 @@ lav_bootstrap_internal <- function(object          = NULL,
         lavoptions$implied <- FALSE
         lavoptions$store.vcov <- FALSE
         lavoptions$se <- "none"
+        if(FUN.orig == "coef") {
+            lavoptions$test <- "none"
+        }
     }
 
     # bollen.stine, yuan, or parametric: we need the Sigma.hat values
@@ -267,7 +270,7 @@ lav_bootstrap_internal <- function(object          = NULL,
             df <- object@test[[1]]$df
             Sigmahat <- Sigma.hat[[g]]
             nmv <- nrow(Sigmahat)
-            n <- lavdata@nobs[[g]]
+            n <- nrow(lavdata@X[[g]])
 
             # Calculate tauhat_1, middle p. 267.
             # Yuan et al note that tauhat_1 could be negative;
@@ -315,8 +318,8 @@ lav_bootstrap_internal <- function(object          = NULL,
             # Note: we generate the bootstrap indices separately for each
             #       group, in order to ensure the group sizes do not change!
             for(g in 1:lavdata@ngroups) {
-                stopifnot(lavdata@nobs[[g]] > 1L)
-                boot.idx <- sample.int(lavdata@nobs[[g]], replace = TRUE)
+                stopifnot(nrow(lavdata@X[[g]]) > 1L)
+                boot.idx <- sample.int(nrow(lavdata@X[[g]]), replace = TRUE)
                 BOOT.idx[[g]] <- boot.idx
                 dataX[[g]] <- dataX[[g]][boot.idx,,drop = FALSE]
             }
@@ -496,7 +499,7 @@ lav_bootstrap_internal <- function(object          = NULL,
     }
 
     # fill in container
-    t.star <- do.call("rbind", res)
+    t.star[] <- do.call("rbind", res)
 
     # handle errors
     error.idx <- which(sapply(res, function(x) is.na(x[1L])))
