@@ -1,4 +1,3 @@
-# constructor for the 'lavData' class
 #
 # the lavData class describes how the data looks like
 #  - do we have a full data frame, or only sample statistics?
@@ -49,6 +48,19 @@ lavData <- function(data              = NULL,          # data.frame
         level.label <- character(0L)
     }
 
+    # block.labels
+    block.label <- character(0L)
+    if(length(group.label) > 0L && length(level.label) == 0L) {
+        block.label <- group.label
+    } else if(length(level.label) > 0L && length(group.label) == 0L) {
+        block.label <- level.label
+    } else if(length(group.label) > 0L &&
+              length(level.label) > 0L) {
+        block.label <- paste(rep(group.label, each = length(level.label)),
+                             rep(level.label, times = length(group.label)),
+                             sep = ".")
+    }
+
     # std.ov?
     std.ov <- lavoptions$std.ov
     if(is.null(std.ov)) {
@@ -126,6 +138,7 @@ lavData <- function(data              = NULL,          # data.frame
                                  cluster           = cluster,
                                  group.label       = group.label,
                                  level.label       = level.label,
+                                 block.label       = block.label,
                                  ov.names          = ov.names,
                                  ordered           = ordered,
                                  sampling.weights  = sampling.weights,
@@ -263,6 +276,19 @@ lavData <- function(data              = NULL,          # data.frame
             }
         }
 
+        # block.labels
+        block.label <- character(0L)
+        if(length(group.label) > 0L && length(level.label) == 0L) {
+            block.label <- group.label
+        } else if(length(level.label) > 0L && length(group.label) == 0L) {
+            block.label <- level.label
+        } else if(length(group.label) > 0L &&
+                  length(level.label) > 0L) {
+            block.label <- paste(rep(group.label, each = length(level.label)),
+                                 rep(level.label, times = length(group.label)),
+                                 sep = ".")
+        }
+
         # construct lavData object
         lavData <- new("lavData",
                        data.type   = "moment",
@@ -272,6 +298,7 @@ lavData <- function(data              = NULL,          # data.frame
                        cluster     = character(0L),
                        group.label = group.label,
                        level.label = character(0L),
+                       block.label = block.label,
                        nobs        = as.list(sample.nobs),
                        norig       = as.list(sample.nobs),
                        ov.names    = ov.names,
@@ -395,10 +422,24 @@ lavData <- function(data              = NULL,          # data.frame
                                                  cluster = cluster,
                                                  multilevel = TRUE,
                                                  ov.names = OV.NAMES,
+                                                 ov.names.x = ov.names.x[[g]],
                                                  ov.names.l = ov.names.l[[g]])
             }
         } # g
 
+        # block.labels
+        block.label <- character(0L)
+        if(length(group.label) > 0L && length(level.label) == 0L) {
+            block.label <- group.label
+        } else if(length(level.label) > 0L && length(group.label) == 0L) {
+            block.label <- level.label
+        } else if(length(group.label) > 0L &&
+                  length(level.label) > 0L) {
+            block.label <- paste(rep(group.label, each = length(level.label)),
+                                 rep(level.label, times = length(group.label)),
+                                 sep = ".")
+        }
+
         # construct lavData object
         lavData <- new("lavData",
                        data.type   = "none",
@@ -408,6 +449,7 @@ lavData <- function(data              = NULL,          # data.frame
                        cluster     = cluster,
                        group.label = group.label,
                        level.label = level.label,
+                       block.label = block.label,
                        nobs        = sample.nobs,
                        norig       = sample.nobs,
                        ov.names    = ov.names,
@@ -437,6 +479,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                           cluster       = NULL,          # clustered?
                           group.label   = NULL,          # custom group labels?
                           level.label   = NULL,
+                          block.label   = NULL,
                           ov.names      = NULL,          # variables needed
                                                          # in model
                           ordered       = NULL,          # ordered variables
@@ -602,7 +645,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
     for(g in 1:ngroups) {
         # does the data contain all the observed variables
         # needed in the user-specified model for this group
-        ov.all <- unique(ov.names[[g]], ov.names.x[[g]]) # no overlap if categ
+        ov.all <- unique(c(ov.names[[g]], ov.names.x[[g]])) # no overlap if categ
 
         # handle interactions
         ov.int.names <- ov.all[ grepl(":", ov.all) ]
@@ -626,7 +669,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
         idx.missing <- which(!(ov.all %in% names(data)))
 
         if(length(idx.missing)) {
-            stop("lavaan ERROR: missing observed variables in dataset: ",
+            stop("lavaan ERROR: some (observed) variables specified in the model are not found in the dataset: ",
                  paste(ov.all[idx.missing], collapse=" "))
         }
     }
@@ -743,8 +786,9 @@ lav_data_full <- function(data          = NULL,          # data.frame
     for(g in 1:ngroups) {
 
         # extract variables in correct order
-        if(!is.null(cluster) && length(cluster) > 0L) {
+        if(nlevels > 1L) {
             # keep 'joint' (Y,X) matrix in @X if multilevel (or always?)
+            # yes for multilevel (for now); no for clustered only
             OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
             ov.idx  <- ov$idx[match(OV.NAMES,   ov$name)]
         } else {
@@ -832,9 +876,12 @@ lav_data_full <- function(data          = NULL,          # data.frame
         user.ordered.names <- ov$name[ov$type == "ordered" & ov$user == 1L]
         user.ordered.idx <- which(ov.names[[g]] %in% user.ordered.names)
         if(length(user.ordered.idx) > 0L) {
-            for(i in user.ordered.idx) {
+          for(i in user.ordered.idx) {
+                X[[g]][,i][is.na(X[[g]][,i])] <- NA # change NaN to NA
                 X[[g]][,i] <- as.numeric(as.factor(X[[g]][,i]))
-            }
+                # possible alternative to the previous two lines:
+                # X[[g]][,i] <- as.numeric(factor(X[[g]][,i], exclude = c(NA, NaN)))
+          }
         }
 
         ## FIXME:
@@ -860,26 +907,6 @@ lav_data_full <- function(data          = NULL,          # data.frame
             }
         }
 
-        # missing data
-        if(missing != "listwise") {
-            # get missing patterns
-            Mp[[g]] <- lav_data_missing_patterns(X[[g]],
-                           sort.freq = TRUE, coverage = TRUE)
-            # checking!
-            if(length(Mp[[g]]$empty.idx) > 0L) {
-                # new in 0.6-4: we return 'original' index in full data.frame
-                empty.case.idx <- case.idx[[g]][ Mp[[g]]$empty.idx ]
-                if(warn) {
-                    warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
-                }
-            }
-            if(warn && any(Mp[[g]]$coverage < 0.1)) {
-                warning("lavaan WARNING: due to missing values, some pairwise combinations have less than 10% coverage")
-            }
-            # in case we had observations with only missings
-            nobs[[g]] <- NROW(X[[g]]) - length(Mp[[g]]$empty.idx)
-        }
-
         # response patterns (ordered variables only)
         ord.idx <- which(ov.names[[g]] %in% ov$name[ov$type == "ordered"])
         if(length(ord.idx) > 0L) {
@@ -894,6 +921,43 @@ lav_data_full <- function(data          = NULL,          # data.frame
             warning("lavaan WARNING: small number of observations (nobs < nvar)", txt,
                     "\n  nobs = ", nobs[[g]], " nvar = ", nvar)
         }
+        # check variances per group (if we have multiple groups)
+        # to catch zero-variance variables within a group (new in 0.6-8)
+        if(ngroups > 1L) {
+            # X
+            group.var <- apply(X[[g]], 2, var, na.rm = TRUE)
+            zero.var <- which(group.var < .Machine$double.eps)
+            if(length(zero.var) == 0L) {
+                # all is good
+            } else {
+                # some zero variances!
+                gtxt <- if(ngroups > 1L) {
+                            paste(" in group ", g, ":", sep = "")
+                        } else { ":" }
+                txt <- c("some variables have no variance ", gtxt,
+                   "\n",
+                   paste( ov.names[[g]][zero.var], collapse = " "))
+                stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+            }
+
+            # eXo (if conditional.x = TRUE)...
+            if(length(exo.idx) > 0L) {
+                group.var <- apply(eXo[[g]], 2, var, na.rm = TRUE)
+                zero.var <- which(group.var < .Machine$double.eps)
+                if(length(zero.var) == 0L) {
+                    # all is good
+                } else {
+                    # some zero variances!
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ":", sep = "")
+                            } else { ":" }
+                    txt <- c("some exogenous variables have no variance ", gtxt,
+                       "\n",
+                       paste( ov.names.x[[g]][zero.var], collapse = " "))
+                    stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+                }
+            }
+        }
 
         # cluster information
         if(length(cluster) > 0L) {
@@ -910,6 +974,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                                                  cluster = cluster,
                                                  multilevel = multilevel,
                                                  ov.names = OV.NAMES,
+                                                 ov.names.x = ov.names.x[[g]],
                                                  ov.names.l = ov.names.l[[g]])
 
             # new in 0.6-4
@@ -998,6 +1063,41 @@ lav_data_full <- function(data          = NULL,          # data.frame
 
         } # clustered data
 
+        # missing data
+        if(missing != "listwise") {
+
+            if(length(cluster) > 0L) {
+                # get missing patterns
+                Mp[[g]] <- lav_data_missing_patterns(X[[g]],
+                               sort.freq = TRUE, coverage = TRUE,
+                               Lp = Lp[[g]])
+            } else {
+                # get missing patterns
+                Mp[[g]] <- lav_data_missing_patterns(X[[g]],
+                               sort.freq = TRUE, coverage = TRUE,
+                               Lp = NULL)
+            }
+
+            # checking!
+            if(length(Mp[[g]]$empty.idx) > 0L) {
+                # new in 0.6-4: return 'original' index in full data.frame
+                empty.case.idx <- case.idx[[g]][ Mp[[g]]$empty.idx ]
+                if(warn) {
+                    warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
+                }
+            }
+            if(warn && any(Mp[[g]]$coverage == 0)) {
+                txt <- c("due to missing values, some pairwise combinations have 0% coverage;", " use lavInspect(fit, \"coverage\") to investigate.")
+                warning(lav_txt2message(txt))
+            } else if(warn && any(Mp[[g]]$coverage < 0.1)) {
+                txt <- c("due to missing values, some pairwise combinations have less than 10% coverage;", " use lavInspect(fit, \"coverage\") to investigate.")
+                warning(lav_txt2message(txt))
+            }
+            # in case we had observations with only missings
+            nobs[[g]] <- NROW(X[[g]]) - length(Mp[[g]]$empty.idx)
+    } # missing
+
+
     } # groups, at first level
 
     # sampling weigths, again
@@ -1026,6 +1126,19 @@ lav_data_full <- function(data          = NULL,          # data.frame
         }
     }
 
+    # block.labels
+    block.label <- character(0L)
+    if(length(group.label) > 0L && length(level.label) == 0L) {
+        block.label <- group.label
+    } else if(length(level.label) > 0L && length(group.label) == 0L) {
+        block.label <- level.label
+    } else if(length(group.label) > 0L &&
+              length(level.label) > 0L) {
+        block.label <- paste(rep(group.label, each = length(level.label)),
+                             rep(level.label, times = length(group.label)),
+                             sep = ".")
+    }
+
 
     lavData <- new("lavData",
                    data.type       = "full",
@@ -1035,6 +1148,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
                    cluster         = cluster,
                    group.label     = group.label,
                    level.label     = level.label,
+                   block.label     = block.label,
                    std.ov          = std.ov,
                    nobs            = nobs,
                    norig           = norig,
@@ -1058,353 +1172,5 @@ lav_data_full <- function(data          = NULL,          # data.frame
     lavData
 }
 
-# get missing patterns
-lav_data_missing_patterns <- function(Y, sort.freq = FALSE, coverage = FALSE) {
 
-    # construct TRUE/FALSE matrix: TRUE if value is observed
-    OBS <- !is.na(Y)
-
-    # empty cases
-    empty.idx <- which(rowSums(OBS) == 0L)
-
-    # this is what we did in < 0.6
-    #if(length(empty.idx) > 0L) {
-    #    OBS <- OBS[-empty.idx,,drop = FALSE]
-    #}
-
-    # pattern of observed values per observation
-    case.id <- apply(1L * OBS, 1L, paste, collapse = "")
-
-    # remove empty patterns
-    if(length(empty.idx)) {
-        case.id.nonempty <- case.id[-empty.idx]
-    } else {
-        case.id.nonempty <- case.id
-    }
-
-    # sort non-empty patterns (from high occurence to low occurence)
-    if(sort.freq) {
-        TABLE <- sort(table(case.id.nonempty), decreasing = TRUE)
-    } else {
-        TABLE <- table(case.id.nonempty)
-    }
-
-    # unique pattern ids
-    pat.id <- names(TABLE)
-
-    # number of patterns
-    pat.npatterns  <- length(pat.id)
-
-    # case idx per pattern
-    pat.case.idx <- lapply(seq_len(pat.npatterns),
-                           function(p) which(case.id == pat.id[p]))
-
-    # unique pattern frequencies
-    pat.freq <- as.integer(TABLE)
-
-    # first occurrence of each pattern
-    pat.first <- match(pat.id, case.id)
-
-    # TRUE/FALSE for each pattern
-    pat.obs <- OBS[pat.first,,drop = FALSE] # observed per pattern
-
-    Mp <- list(npatterns = pat.npatterns, id = pat.id, freq = pat.freq,
-               case.idx = pat.case.idx, pat = pat.obs, empty.idx = empty.idx)
-
-    if(coverage) {
-        # FIXME: if we have empty cases, include them in N?
-        # no for now
-        Mp$coverage <- crossprod(OBS) / sum(pat.freq)
-        #Mp$coverage <- crossprod(OBS) / NROW(Y)
-    }
-
-    Mp
-}
-
-# get response patterns (ignore empty cases!)
-lav_data_resp_patterns <- function(Y) {
-
-    # construct TRUE/FALSE matrix: TRUE if value is observed
-    OBS <- !is.na(Y)
-
-    # empty cases
-    empty.idx <- which(rowSums(OBS) == 0L)
-
-    # removeYempty cases
-    if(length(empty.idx) > 0L) {
-        Y <- Y[-empty.idx,,drop = FALSE]
-    }
-
-    ntotal <- nrow(Y); nvar <- ncol(Y)
-
-    # identify, label and sort response patterns
-    id <- apply(Y, MARGIN = 1, paste, collapse = "")
-
-    # sort patterns (from high occurence to low occurence)
-    TABLE <- sort(table(id), decreasing = TRUE)
-    order <- names(TABLE)
-    npatterns <- length(TABLE)
-    pat <- Y[match(order, id), , drop = FALSE]
-    row.names(pat) <- as.character(TABLE)
-
-    # handle NA?
-    Y[is.na(Y)] <- -9
-    total.patterns <- prod(apply(Y, 2, function(x) length(unique(x))))
-    empty.patterns <- total.patterns - npatterns
-    # return a list
-    #out <- list(nobs=ntotal, nvar=nvar,
-    #            id=id, npatterns=npatterns,
-    #            order=order, pat=pat)
-
-    # only return pat
-    out <- list(npatterns=npatterns, pat=pat, total.patterns=total.patterns,
-                empty.patterns=empty.patterns)
-
-    out
-}
-
-# get cluster information
-# - cluster can be a vector!
-# - clus can contain multiple columns!
-lav_data_cluster_patterns <- function(Y = NULL,
-                                      clus = NULL,    # the cluster ids
-                                      cluster = NULL, # the cluster 'names'
-                                      multilevel = FALSE,
-                                      ov.names, ov.names.l) {
-
-    # how many levels?
-    nlevels <- length(cluster) + 1L
-
-    # did we get any data (or is this just for simulateData)
-    if(!is.null(Y) && !is.null(clus)) {
-        haveData <- TRUE
-    } else {
-        haveData <- FALSE
-    }
-
-    # check clus
-    if(haveData) {
-        stopifnot(ncol(clus) == (nlevels - 1L), nrow(Y) == nrow(clus))
-    }
-
-    cluster.size    <- vector("list", length = nlevels)
-    cluster.id      <- vector("list", length = nlevels)
-    cluster.idx     <- vector("list", length = nlevels)
-    nclusters       <- vector("list", length = nlevels)
-    cluster.sizes   <- vector("list", length = nlevels)
-    ncluster.sizes  <- vector("list", length = nlevels)
-    cluster.size.ns <- vector("list", length = nlevels)
-    ov.idx          <- vector("list", length = nlevels)
-    both.idx        <- vector("list", length = nlevels)
-    within.idx      <- vector("list", length = nlevels)
-    between.idx     <- vector("list", length = nlevels)
-    both.names      <- vector("list", length = nlevels)
-    within.names    <- vector("list", length = nlevels)
-    between.names   <- vector("list", length = nlevels)
-
-    # level-1 is special
-    if(haveData) {
-        nclusters[[1]] <- NROW(Y)
-    }
-
-    if(multilevel) {
-        ov.idx[[1]] <- match(ov.names.l[[1]], ov.names)
-    }
-
-    # for the remaining levels...
-    for(l in 2:nlevels) {
-        if(haveData) {
-            CLUS <- clus[,(l-1L)]
-            cluster.id[[l]]      <- unique(CLUS)
-            cluster.idx[[l]]     <- match(CLUS, cluster.id[[l]])
-            cluster.size[[l]]    <- tabulate(cluster.idx[[l]])
-            nclusters[[l]]       <- length(cluster.size[[l]])
-            cluster.sizes[[l]]   <- unique(cluster.size[[l]])
-            ncluster.sizes[[l]]  <- length(cluster.sizes[[l]])
-            cluster.size.ns[[l]] <- as.integer(table(factor(cluster.size[[l]],
-                                     levels = as.character(cluster.sizes[[l]]))))
-        } else {
-            cluster.id[[l]]      <- integer(0L)
-            cluster.idx[[l]]     <- integer(0L)
-            cluster.size[[l]]    <- integer(0L)
-            nclusters[[l]]       <- integer(0L)
-            cluster.sizes[[l]]   <- integer(0L)
-            ncluster.sizes[[l]]  <- integer(0L)
-            cluster.size.ns[[l]] <- integer(0L)
-        }
-
-        if(multilevel) {
-            # index of ov.names for this level
-            ov.idx[[l]]         <- match(ov.names.l[[l]], ov.names)
-
-            both.idx[[l]]       <- which( ov.names %in% ov.names.l[[1]] &
-                                          ov.names %in% ov.names.l[[2]])
-            within.idx[[l]]     <- which( ov.names %in% ov.names.l[[1]] &
-                                         !ov.names %in% ov.names.l[[2]])
-            between.idx[[l]]    <- which(!ov.names %in% ov.names.l[[1]] &
-                                          ov.names %in% ov.names.l[[2]])
-
-            # names
-            both.names[[l]]     <- ov.names[ ov.names %in% ov.names.l[[1]] &
-                                             ov.names %in% ov.names.l[[2]] ]
-            within.names[[l]]   <- ov.names[ ov.names %in% ov.names.l[[1]] &
-                                            !ov.names %in% ov.names.l[[2]] ]
-            between.names[[l]]  <- ov.names[!ov.names %in% ov.names.l[[1]] &
-                                             ov.names %in% ov.names.l[[2]] ]
-        }
-    }
-
-    out <- list(cluster = cluster, # clus = clus,
-                # per level
-                nclusters = nclusters,
-                cluster.size = cluster.size, cluster.id = cluster.id,
-                cluster.idx = cluster.idx, cluster.sizes = cluster.sizes,
-                ncluster.sizes = ncluster.sizes,
-                cluster.size.ns = cluster.size.ns,
-                ov.idx = ov.idx, both.idx = both.idx, within.idx = within.idx,
-                between.idx = between.idx,
-                both.names = both.names, within.names = within.names,
-                between.names = between.names)
-
-    out
-}
-
-setMethod("show", "lavData",
-function(object) {
-    # print 'lavData' object
-    lav_data_print_short(object)
-})
-
-lav_data_print_short <- function(object, nd = 3L) {
-
-    lavdata <- object
-    num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
-
-    # listwise deletion?
-    listwise <- twocolumn <- FALSE
-    for(g in 1:lavdata@ngroups) {
-       if(lavdata@nobs[[1L]] != lavdata@norig[[1L]]) {
-           listwise <- twocolumn <- TRUE
-           break
-       }
-    }
-
-    # header? no, for historical reasons only
-    #cat("Data information:\n\n")
-
-    c1 <- c2 <- c3 <- character(0L)
-
-    # number of observations
-    if(lavdata@ngroups == 1L) {
-        if(listwise) {
-            c1 <- c(c1, ""); c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
-        }
-        c1 <- c(c1, "Number of observations")
-        c2 <- c(c2, lavdata@nobs[[1L]])
-        c3 <- c(c3, ifelse(listwise, lavdata@norig[[1L]], ""))
-    } else {
-        c1 <- c(c1, "Number of observations per group:");
-        if(listwise) {
-            c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
-        } else {
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-        }
-        for(g in 1:lavdata@ngroups) {
-            c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-            c2 <- c(c2, lavdata@nobs[[g]])
-            c3 <- c(c3, ifelse(listwise, lavdata@norig[[g]], ""))
-        } # g
-    }
-
-    # number of clusters
-    if(lavdata@ngroups == 1L) {
-        if( (.hasSlot(lavdata, "nlevels")) && # in case we have an old obj
-            (lavdata@nlevels > 1L) ) {
-            for(l in 2:lavdata@nlevels) {
-                c1 <- c(c1,
-                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                        sep = ""))
-                c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[l]])
-                c3 <- c(c3, "")
-            }
-        } else if( (.hasSlot(lavdata, "cluster")) &&
-                   (length(lavdata@cluster) > 0L) ) {
-            c1 <- c(c1, paste("Number of clusters [", lavdata@cluster, "]",
-                              sep = ""))
-            c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[2]])
-            c3 <- c(c3, "")
-        }
-    } else {
-        if( (.hasSlot(lavdata, "nlevels")) && (lavdata@nlevels > 1L) ) {
-            for(l in 2:lavdata@nlevels) {
-                c1 <- c(c1,
-                  paste("Number of clusters [", lavdata@cluster[l-1], "]:",
-                        sep = ""))
-                c2 <- c(c2, ""); c3 <- c(c3, "")
-                for(g in 1:lavdata@ngroups) {
-                    c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                    c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[l]])
-                    c3 <- c(c3, "")
-                }
-            }
-        } else if( (.hasSlot(lavdata, "cluster")) &&
-               (length(lavdata@cluster) > 0L) ) {
-            c1 <- c(c1,
-             paste("Number of clusters [", lavdata@cluster, "]:", sep = ""))
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-            for(g in 1:lavdata@ngroups) {
-                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[2]])
-                c3 <- c(c3, "")
-            }
-        }
-    }
-
-    # missing patterns?
-    if(!is.null(lavdata@Mp[[1L]])) {
-        if(lavdata@ngroups == 1L) {
-            c1 <- c(c1, "Number of missing patterns")
-            c2 <- c(c2, lavdata@Mp[[1L]]$npatterns)
-            c3 <- c(c3, "")
-        } else {
-            c1 <- c(c1, "Number of missing patterns per group:")
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-            for(g in 1:lavdata@ngroups) {
-                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                c2 <- c(c2, lavdata@Mp[[g]]$npatterns)
-                c3 <- c(c3, "")
-            }
-        }
-    }
-
-    # sampling weights?
-    if( (.hasSlot(lavdata, "weights")) && # in case we have an old object
-        (!is.null(lavdata@weights[[1L]])) ) {
-        c1 <- c(c1, "Sampling weights variable")
-        c2 <- c(c2, lavdata@sampling.weights)
-        c3 <- c(c3, "")
-    }
-
-    # empty row
-    c1 <- c(c1, ""); c2 <- c(c2, ""); c3 <- c(c3, "")
-
-    # format c1/c2
-    c1 <- format(c1, width = 43L)
-    c2 <- format(c2, width = 8L + max(0, (nd - 3L)) * 4L, justify = "right")
-    c3 <- format(c3, width = 8L + nd, justify = "right")
-
-    # create character matrix
-    if(twocolumn) {
-        M <- cbind(c1, c2, c3, deparse.level = 0)
-    } else {
-        M <- cbind(c1, c2, deparse.level = 0)
-    }
-    colnames(M) <- rep("",  ncol(M))
-    rownames(M) <- rep(" ", nrow(M))
-
-    # print
-    write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
-
-    invisible(M)
-}
 
