@@ -70,8 +70,20 @@ lav_model_estimate <- function(lavmodel       = NULL,
 
     if(lavoptions$optim.parscale == "none") {
         # do nothing, but still set SCALE, as before
+
+
+    # 0.6-17:
+    # only temporarily: 'keep' this mistake, and change it later:
+    # (note the "standarized")
+    # we only do this to avoid breaking a test in semlbci
     } else if(lavoptions$optim.parscale %in% c("stand", "st", "standardize",
                                                "standarized", "stand.all")) {
+
+    # this is what it should be:
+    # } else if(lavoptions$optim.parscale %in% c("stand", "st", "standardize",
+    #                                            "standardized", "stand.all")) {
+
+
         # rescale parameters as if the data was standardized
         # new in 0.6-2
         #
@@ -426,9 +438,10 @@ lav_model_estimate <- function(lavmodel       = NULL,
             if(!attr(Sigma.hat[[g]], "po")) {
                 group.txt <- ifelse(ngroups > 1,
                                     paste(" in group ",g,".",sep=""), ".")
-                if(debug) print(Sigma.hat[[g]])
-                print(Sigma.hat[[g]][,])
-                stop("lavaan ERROR: initial model-implied matrix (Sigma) is not positive definite;\n  check your model and/or starting parameters", group.txt)
+                if(debug) {
+                    print(Sigma.hat[[g]][,])
+                }
+                warning("lavaan WARNING: initial model-implied matrix (Sigma) is not positive definite;\n  check your model and/or starting parameters", group.txt)
                 x <- start.x
                 fx <- as.numeric(NA)
                 attr(fx, "fx.group") <- rep(as.numeric(NA), ngroups)
@@ -730,7 +743,11 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                trace=0L,
                                #abs.tol=1e-20,
                                abs.tol=(.Machine$double.eps * 10),
-                               rel.tol=1e-9) # 1e-10 seems 'too strict'
+                               rel.tol=1e-9, # 1e-10 seems 'too strict'
+                               step.min=1.0, # 1.0 in < 0.5-21
+                               step.max=1.0,
+                               x.tol=1.5e-8,
+                               xf.tol=2.2e-14)
         control.nlminb <- modifyList(control.nlminb, lavoptions$control)
         control <- control.nlminb[c("eval.max", "iter.max", "trace",
                                     "abs.tol", "rel.tol")]
