@@ -18,6 +18,9 @@ fml_deriv1 <- function(Sigma.hat = NULL,    # model-based var/cov/cor
 # chain rule to get the gradient
 # this is adapted from code written by Myrsini Katsikatsou
 # first attempt - YR 5 okt 2012
+# HJ 18/10/23: Modification for complex design and completely observed data (no
+# missing) with only ordinal indicators to get the right gradient for the
+# optimisation and Hessian computation.
 pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
                        Mu.hat     = NULL,       # model-based means
                        TH         = NULL,       # model-based thresholds + means
@@ -25,6 +28,7 @@ pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
                        num.idx    = NULL,       # which variables are numeric
                        X          = NULL,       # data
                        eXo        = NULL,       # external covariates
+                       wt         = NULL,       # case weights (not used yet)
                        lavcache   = NULL,       # housekeeping stuff
                        PI         = NULL,       # slopes
                        missing    = "listwise", # how to deal with missings
@@ -82,12 +86,21 @@ pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
 
     # shortcut for ordinal-only/no-exo case
     if(!scores && all(ov.types == "ordered") && nexo == 0L) {
-        gradient <- grad_tau_rho(no.x               = nvar,
-                                 all.thres          = TH,
-                                 index.var.of.thres = th.idx,
-                                 rho.xixj           = cors,
-                                 n.xixj.vec         = lavcache$bifreq,
-                                 out.LongVecInd     = lavcache$LONG)
+      # >>>>>>>> HJ/MK PML CODE >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+
+      if (is.null(wt)) {
+        n.xixj.vec <- lavcache$bifreq
+      } else {
+        n.xixj.vec <- lavcache$sum_obs_weights_xixj_ab_vec
+      }
+      gradient <- grad_tau_rho(no.x               = nvar,
+                               all.thres          = TH,
+                               index.var.of.thres = th.idx,
+                               rho.xixj           = cors,
+                               n.xixj.vec         = n.xixj.vec,
+                               out.LongVecInd     = lavcache$LONG)
+
+      # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 
         if(missing == "available.cases") {
             uniPI <- univariateExpProbVec(TH = TH, th.idx = th.idx)
@@ -200,10 +213,12 @@ pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
                     stop("lavaan ERROR: mixed + exo in PML not implemented; try optim.gradient = \"numerical\"")
                 }
 
-                SC.COR.UNI <- ps_cor_scores_no_exo(Y1 = X[,i], Y2 = X[,j],
-                                                   var.y1 = Sigma.hat[i,i],
-                                                   eta.y1 = rep(Mu.hat[i], N),
+                SC.COR.UNI <- lav_bvmix_cor_scores(Y1 = X[,i], Y2 = X[,j],
+                                                   eXo = NULL, wt = wt,
+                                                   evar.y1 = Sigma.hat[i,i],
+                                                   beta.y1 = Mu.hat[i],
                                                    th.y2 = TH[ th.idx == j ],
+                                                   sl.y2 = NULL,
                                                    rho = Cor.hat[i,j],
                                                    sigma.correction = TRUE)
 
@@ -245,9 +260,10 @@ pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
                     stop("lavaan ERROR: mixed + exo in PML not implemented; try optim.gradient = \"numerical\"")
                 }
 
-                SC.COR.UNI <- ps_cor_scores_no_exo(Y1 = X[,j], Y2 = X[,i],
-                                                   var.y1 = Sigma.hat[j,j],
-                                                   eta.y1 = rep(Mu.hat[j], N),
+                SC.COR.UNI <- lav_bvmix_cor_scores(Y1 = X[,j], Y2 = X[,i],
+                                                   eXo = NULL, wt = wt,
+                                                   evar.y1 = Sigma.hat[j,j],
+                                                   beta.y1 = Mu.hat[j],
                                                    th.y2 = TH[ th.idx == i ],
                                                    rho = Cor.hat[i,j],
                                                    sigma.correction = TRUE)
@@ -287,17 +303,17 @@ pml_deriv1 <- function(Sigma.hat  = NULL,       # model-based var/cov/cor
             } else if(ov.types[i] == "ordered" && ov.types[j] == "ordered") {
                 # polychoric correlation
                 if(nexo == 0L) {
-                    SC.COR.UNI <- pc_cor_scores(Y1  = X[,i],
-                                                Y2  = X[,j],
-                                                eXo = NULL,
-                                                rho = Sigma.hat[i,j],
-                                                fit.y1 = NULL, # fixme
-                                                fit.y2 = NULL, # fixme
-                                                th.y1 = TH[ th.idx == i ],
-                                                th.y2 = TH[ th.idx == j ],
-                                                sl.y1 = NULL,
-                                                sl.y2 = NULL,
-                                                na.zero = TRUE)
+                    SC.COR.UNI <-
+                        lav_bvord_cor_scores(Y1  = X[,i], Y2  = X[,j],
+                                             eXo = NULL, wt = wt,
+                                             rho = Sigma.hat[i,j],
+                                             fit.y1 = NULL, # fixme
+                                             fit.y2 = NULL, # fixme
+                                             th.y1 = TH[ th.idx == i ],
+                                             th.y2 = TH[ th.idx == j ],
+                                             sl.y1 = NULL,
+                                             sl.y2 = NULL,
+                                             na.zero = TRUE)
                 } else {
                     SC.COR.UNI <-
                         pc_cor_scores_PL_with_cov(Y1 = X[,i],
