@@ -18,17 +18,28 @@ lav_model_objective <- function(lavmodel       = NULL,
         return(fx)
     }
 
-    meanstructure <- lavmodel@meanstructure
-    estimator     <- lavmodel@estimator
-    categorical   <- lavmodel@categorical
-    group.w.free  <- lavmodel@group.w.free
-    fixed.x       <- lavmodel@fixed.x
-    conditional.x <- lavmodel@conditional.x
-    num.idx       <- lavmodel@num.idx
-    th.idx        <- lavmodel@th.idx
+    meanstructure  <- lavmodel@meanstructure
+    estimator      <- lavmodel@estimator
+    categorical    <- lavmodel@categorical
+    if(.hasSlot(lavmodel, "correlation")) {
+        correlation    <- lavmodel@correlation
+    } else {
+        correlation    <- FALSE
+    }
+    group.w.free   <- lavmodel@group.w.free
+    fixed.x        <- lavmodel@fixed.x
+    conditional.x  <- lavmodel@conditional.x
+    num.idx        <- lavmodel@num.idx
+    th.idx         <- lavmodel@th.idx
+    if(.hasSlot(lavmodel, "estimator.args")) {
+        estimator.args <- lavmodel@estimator.args
+    } else {
+        estimator.args <- list()
+    }
+
 
     # do we need WLS.est?
-    if(estimator %in% c("ULS", "GLS", "WLS", "DWLS", "NTRLS")) {
+    if(estimator %in% c("ULS", "WLS", "DWLS", "NTRLS", "DLS")) {
 
         lavimplied <- lav_model_implied(lavmodel, GLIST = GLIST)
         # check for COV with negative diagonal elements?
@@ -52,8 +63,13 @@ lav_model_objective <- function(lavmodel       = NULL,
                                          extra = TRUE)
             Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
         }
+        if(estimator == "DLS" && estimator.args$dls.GammaNT == "model") {
+            Sigma.hat <- computeSigmaHat(lavmodel = lavmodel, GLIST = GLIST,
+                                         extra = FALSE)
+            Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
+        }
         if(debug) print(WLS.est)
-    } else if(estimator %in% c("ML", "PML", "FML", "REML") &&
+    } else if(estimator %in% c("ML", "GLS", "PML", "FML", "REML", "catML") &&
               lavdata@nlevels == 1L) {
         # compute moments for all groups
         #if(conditional.x) {
@@ -62,7 +78,8 @@ lav_model_objective <- function(lavmodel       = NULL,
         #                     extra = (estimator %in% c("ML", "REML","NTRLS")))
         #} else {
             Sigma.hat <- computeSigmaHat(lavmodel = lavmodel, GLIST = GLIST,
-                             extra = (estimator %in% c("ML", "REML","NTRLS")))
+                             extra = (estimator %in% c("ML", "REML",
+                                                       "NTRLS", "catML")))
         #}
 
         if(estimator == "REML") {
@@ -120,23 +137,30 @@ lav_model_objective <- function(lavmodel       = NULL,
                                            Yp=lavsamplestats@missing[[g]],
                                            h1=lavsamplestats@missing.h1[[g]]$h1,                                           N=lavsamplestats@nobs[[g]])
             } else if(estimator == "ML" && lavdata@nlevels > 1L) {
-                #group.fx <- estimator.2L(lavmodel       = lavmodel,
-                #                         GLIST          = GLIST,
-                #                         lavdata        = lavdata,
-                #                         lavsamplestats = lavsamplestats,
-                #                         group          = g)
-                group.fx <- 0
+                # FIML twolevel
+                group.fx <- estimator.2L(lavmodel       = lavmodel,
+                                         GLIST          = GLIST,
+                                         Y1             = lavdata@X[[g]],
+                                         Lp             = lavdata@Lp[[g]],
+                                         Mp             = lavdata@Mp[[g]],
+                                         lavsamplestats = lavsamplestats,
+                                         group          = g)
             } else {
                 stop("this estimator: `", estimator,
                      "' can not be used with incomplete data and the missing=\"ml\" option")
             }
-        } else if(estimator == "ML" || estimator == "Bayes") {
+        } else if(estimator == "ML" || estimator == "Bayes" ||
+                  estimator == "catML") {
         # complete data
             # ML and friends
             if(lavdata@nlevels > 1L) {
+                if(estimator %in% c("catML", "Bayes")) {
+                    stop("lavaan ERROR: multilevel data not supported for estimator ", estimator)
+                }
                 group.fx <- estimator.2L(lavmodel       = lavmodel,
                                          GLIST          = GLIST,
                                          Lp             = lavdata@Lp[[g]],
+                                         Mp             = NULL, # complete data
                                          lavsamplestats = lavsamplestats,
                                          group          = g)
             } else if(conditional.x) {
@@ -159,12 +183,41 @@ lav_model_objective <- function(lavmodel       = NULL,
                     data.cov.log.det = lavsamplestats@cov.log.det[[g]],
                     meanstructure    = meanstructure)
             }
-        } else if(estimator == "GLS" ||
-                  estimator == "WLS" ||
+
+
+        ### GLS #### (0.6-10: nog using WLS function any longer)
+        } else if(estimator == "GLS") {
+            group.fx <- estimator.GLS(
+                    Sigma.hat        = Sigma.hat[[g]],
+                    Mu.hat           = Mu.hat[[g]],
+                    data.cov         = lavsamplestats@cov[[g]],
+                    data.cov.inv     = lavsamplestats@icov[[g]],
+                    data.mean        = lavsamplestats@mean[[g]],
+                    meanstructure    = meanstructure)
+
+        } else if( estimator == "WLS" ||
+                  estimator == "DLS" ||
                   estimator == "NTRLS") {
             # full weight matrix
-            if(estimator == "GLS" || estimator == "WLS") {
+            if(estimator == "WLS") {
                 WLS.V <- lavsamplestats@WLS.V[[g]]
+            } else if(estimator == "DLS") {
+                if(estimator.args$dls.GammaNT == "sample") {
+                    WLS.V <- lavsamplestats@WLS.V[[g]]
+                } else {
+                    dls.a <- estimator.args$dls.a
+                    GammaNT <- lav_samplestats_Gamma_NT(
+                        COV            = Sigma.hat[[g]],
+                        MEAN           = Mu.hat[[g]],
+                        rescale        = FALSE,
+                        x.idx          = lavsamplestats@x.idx[[g]],
+                        fixed.x        = lavmodel@fixed.x,
+                        conditional.x  = lavmodel@conditional.x,
+                        meanstructure  = lavmodel@meanstructure,
+                        slopestructure = lavmodel@conditional.x)
+                    W.DLS <- (1 - dls.a)*lavsamplestats@NACOV[[g]] + dls.a*GammaNT
+                    WLS.V <- lav_matrix_symmetric_inverse(W.DLS)
+                }
             } else if(estimator == "NTRLS") {
                 #WLS.V <- lav_samplestats_Gamma_inverse_NT(
                 #             ICOV = attr(Sigma.hat[[g]],"inv")[,,drop=FALSE],
@@ -214,6 +267,7 @@ lav_model_objective <- function(lavmodel       = NULL,
                                           num.idx   = num.idx[[g]],
                                           X         = lavdata@X[[g]],
                                           eXo       = lavdata@eXo[[g]],
+                                          wt        = lavdata@weights[[g]],
                                           lavcache  = lavcache[[g]],
                                           missing   = lavdata@missing)
             } else {
@@ -225,6 +279,7 @@ lav_model_objective <- function(lavmodel       = NULL,
                                           num.idx   = num.idx[[g]],
                                           X         = lavdata@X[[g]],
                                           eXo       = NULL,
+                                          wt        = lavdata@weights[[g]],
                                           lavcache  = lavcache[[g]],
                                           missing   = lavdata@missing)
             }
@@ -265,13 +320,19 @@ lav_model_objective <- function(lavmodel       = NULL,
             stop("unsupported estimator: ", estimator)
         }
 
-        if(estimator == "ML" || estimator == "REML" || estimator == "NTRLS") {
+        if(estimator %in% c("ML", "REML", "NTRLS", "catML")) {
             if(lavdata@nlevels == 1L) {
                 group.fx <- 0.5 * group.fx ## FIXME
             }
         } else if(estimator == "PML" || estimator == "FML" ||
                   estimator == "MML") {
             # do nothing
+        } else if(estimator == "DLS") {
+            if(estimator.args$dls.FtimesNminus1) {
+                group.fx <- 0.5 * (lavsamplestats@nobs[[g]]-1)/lavsamplestats@nobs[[g]] * group.fx
+            } else {
+                group.fx <- 0.5 * group.fx
+            }
         } else {
             group.fx <- 0.5 * (lavsamplestats@nobs[[g]]-1)/lavsamplestats@nobs[[g]] * group.fx
         }
@@ -301,7 +362,8 @@ lav_model_objective <- function(lavmodel       = NULL,
     }
 
     # penalty for group.w + ML
-    if(group.w.free && estimator %in% c("ML","MML","FML","PML", "REML")) {
+    if(group.w.free && estimator %in% c("ML","MML","FML","PML",
+                                        "REML", "catML")) {
         #obs.prop <- unlist(lavsamplestats@group.w)
         #est.prop <- unlist(GW)
         # if(estimator %in% c("WLS", "GLS", ...) {
@@ -325,7 +387,7 @@ lav_model_objective <- function(lavmodel       = NULL,
 
         # saturated - poisson
         #fx.w <- sum(obs.freq * log(obs.freq/est.freq))
-        # does not work without constraints?
+        # does not work without constraints? --> need lagrange multiplier
 
         fx <- fx + (fx.w + fx.sat)
     }
