@@ -649,6 +649,12 @@ lav_model_vcov_se <- function(lavmodel, lavpartable, VCOV = NULL,
         def.idx <- which(lavpartable$op == ":=")
         if(length(def.idx) > 0L) {
             if(!is.null(BOOT)) {
+                # we must remove the NA rows (and hope we have something left)
+                error.idx <- attr(BOOT, "error.idx")
+                if(length(error.idx) > 0L) {
+                    BOOT <- BOOT[-error.idx,,drop = FALSE] # drops attributes
+                }
+
                 BOOT.def <- apply(BOOT, 1L, lavmodel@def.function)
                 if(length(def.idx) == 1L) {
                     BOOT.def <- as.matrix(BOOT.def)
