@@ -41,23 +41,25 @@ function(object) {
 })
 
 setMethod("summary", "lavaan",
-function(object, header       = TRUE,
-                 fit.measures = FALSE,
-                 estimates    = TRUE,
-                 ci           = FALSE,
-                 fmi          = FALSE,
-                 std          = FALSE,
-                 standardized = FALSE,
-                 remove.step1 = TRUE,
-                 cov.std      = TRUE,
-                 rsquare      = FALSE,
-                 std.nox      = FALSE,
-                 fm.args      = list(standard.test        = "default",
-                                     scaled.test          = "default",
-                                     rmsea.ci.level       = 0.90,
-                                     rmsea.h0.closefit    = 0.05,
-                                     rmsea.h0.notclosefit = 0.08,
-                                     cat.check.pd         = TRUE),
+function(object, header        = TRUE,
+                 fit.measures  = FALSE,
+                 estimates     = TRUE,
+                 ci            = FALSE,
+                 fmi           = FALSE,
+                 std           = FALSE,
+                 standardized  = FALSE,
+                 remove.step1  = TRUE,
+                 remove.unused = TRUE,
+                 cov.std       = TRUE,
+                 rsquare       = FALSE,
+                 std.nox       = FALSE,
+                 fm.args       = list(standard.test        = "default",
+                                      scaled.test          = "default",
+                                      rmsea.ci.level       = 0.90,
+                                      rmsea.h0.closefit    = 0.05,
+                                      rmsea.h0.notclosefit = 0.08,
+                                      robust               = TRUE,
+                                      cat.check.pd         = TRUE),
                  modindices   = FALSE,
                  nd = 3L, cutoff = 0.3, dot.cutoff = 0.1) {
 
@@ -67,7 +69,8 @@ function(object, header       = TRUE,
     res <- lav_object_summary(object = object, header = header,
                fit.measures = fit.measures, estimates = estimates,
                ci = ci, fmi = fmi, std = std, standardized = standardized,
-               remove.step1 = remove.step1, cov.std = cov.std,
+               remove.step1 = remove.step1, remove.unused = remove.unused,
+               cov.std = cov.std,
                rsquare = rsquare, std.nox = std.nox, efa = efa.flag,
                fm.args = fm.args, modindices = modindices)
     # res has class c("lavaan.summary", "list")
@@ -171,7 +174,7 @@ standardizedSolution <-
                                             type = type, free.only = FALSE,
                                             add.labels = FALSE,
                                             add.class = FALSE))
-        if(inherits(VCOV, "try-error")) {
+        if(inherits(VCOV, "try-error") || is.null(VCOV)) {
             LIST$se <- rep(NA, length(LIST$lhs))
             if(zstat) {
                 LIST$z  <- rep(NA, length(LIST$lhs))
@@ -299,7 +302,7 @@ parameterestimates <- function(object,
                                remove.def            = FALSE,
                                remove.nonfree        = FALSE,
                                remove.step1          = TRUE,
-                               #remove.nonfree.scales = FALSE,
+                               remove.unused         = FALSE,
 
                                # output
                                add.attributes = FALSE,
@@ -770,6 +773,15 @@ parameterestimates <- function(object,
                               level = level,
                               est = unlist(r2), stringsAsFactors = FALSE )
             }
+            # add step column if needed
+            if(!is.null(LIST$step)) {
+                R2$step <- 2L # per default
+                # simplification: we assume that only the
+                # observed indicators of latent variables are step 1
+                ov.ind <- unlist(object@pta$vnames$ov.ind)
+                step1.idx <- which(R2$lhs %in% ov.ind)
+                R2$step[step1.idx] <- 1L
+            }
             LIST <- lav_partable_merge(pt1 = LIST, pt2 = R2, warn = FALSE)
         }
     }
@@ -815,7 +827,7 @@ parameterestimates <- function(object,
         SE.comp <- parameterEstimates(fit.complete, ci = FALSE, fmi = FALSE,
             zstat = FALSE, pvalue = FALSE, remove.system.eq = FALSE,
             remove.eq = FALSE, remove.ineq = FALSE,
-            remove.def = FALSE, remove.nonfree = FALSE,
+            remove.def = FALSE, remove.nonfree = FALSE, remove.unused = FALSE,
             rsquare = rsquare, add.attributes = FALSE)$se
 
         SE.comp <- ifelse(SE.comp == 0.0, as.numeric(NA), SE.comp)
@@ -848,17 +860,32 @@ parameterestimates <- function(object,
         }
     }
 
-    # remove non-free scales (categorical only), except 'user-specified'
-    #
-    # (not yet public)
-    #
-    #if(remove.nonfree.scales) {
-    #    nonfree.scales.idx <- which( LIST$free == 0L & LIST$op == "~*~" &
-    #                                 LIST$user == 0L)
-    #    if(length(nonfree.scales.idx) > 0L) {
-    #        LIST <- LIST[-nonfree.scales.idx,]
-    #    }
-    #}
+    # remove 'unused' parameters
+    # these are parameters that are automatically added (user == 0),
+    # but with their final (est) values fixed to their default values
+    # (typically 1 or 0).
+    # currently only intercepts and scaling-factors (for now)
+    # should we also remove fixed-to-1 variances? (parameterization = theta)?
+    if(remove.unused) {
+        # intercepts
+        int.idx <- which(LIST$op == "~1" &
+                         LIST$user == 0L &
+                         LIST$free == 0L &
+                         LIST$est == 0)
+        if(length(int.idx) > 0L) {
+            LIST <- LIST[-int.idx,]
+        }
+
+        # scaling factors
+        scaling.idx <- which(LIST$op == "~*~" &
+                             LIST$user == 0L &
+                             LIST$free == 0L &
+                             LIST$est == 1)
+        if(length(scaling.idx) > 0L) {
+            LIST <- LIST[-scaling.idx,]
+        }
+    }
+
 
     # remove 'free' column
     LIST$free <- NULL
@@ -916,6 +943,8 @@ parameterestimates <- function(object,
         class(LIST) <- c("lavaan.parameterEstimates", "lavaan.data.frame",
                          "data.frame")
         if(header) {
+            attr(LIST, "categorical") <- object@Model@categorical
+            attr(LIST, "parameterization") <- object@Model@parameterization
             attr(LIST, "information") <- object@Options$information[1]
             attr(LIST, "information.meat") <- object@Options$information.meat
             attr(LIST, "se") <- object@Options$se
