@@ -1,3 +1,5 @@
+# add parameter bounds to the parameter table
+# lavoptions$optim.bounds
 lav_partable_add_bounds <- function(partable       = NULL,
                                     lavpta         = NULL,
                                     lavh1          = NULL,
@@ -5,6 +7,11 @@ lav_partable_add_bounds <- function(partable       = NULL,
                                     lavsamplestats = NULL,
                                     lavoptions     = NULL) {
 
+    # no support (yet) for multilevel
+    if(lav_partable_nlevels(partable) > 1L) {
+        return(partable)
+    }
+
     # check optim.bounds
     if(is.null(lavoptions$optim.bounds)) {
         # <0.6-6 version
@@ -47,6 +54,10 @@ lav_partable_add_bounds <- function(partable       = NULL,
             }
         }
 
+        if(is.null(optim.bounds$min.var.ov)) {
+            optim.bounds$min.var.ov <- -Inf
+        }
+
         if(is.null(optim.bounds$min.var.lv.exo)) {
             optim.bounds$min.var.lv.exo <- 0.0
         }
@@ -55,6 +66,10 @@ lav_partable_add_bounds <- function(partable       = NULL,
             optim.bounds$min.var.lv.endo <- 0.0
         }
 
+        if(is.null(optim.bounds$max.r2.lv.endo)) {
+            optim.bounds$max.r2.lv.endo <- 1.0
+        }
+
         if(is.null(optim.bounds$lower.factor)) {
             optim.bounds$lower.factor <- rep(1.0, length(optim.bounds$lower))
         } else {
@@ -87,6 +102,21 @@ lav_partable_add_bounds <- function(partable       = NULL,
 
     }
 
+    # new in 0.6-17: check if we have theta parameterization
+    theta.parameterization.flag <- FALSE
+    if(any(partable$op == "~*~") && lavoptions$parameterization == "theta") {
+        # some fixed-to-1 theta elements?
+        ov.scaled <- partable$lhs[partable$op == "~*~"]
+        ov.var.idx <- which(partable$op == "~~" &
+                            partable$lhs %in% ov.scaled &
+                            partable$free == 0L &
+                            partable$ustart == 1)
+        if(length(ov.var.idx) > 0L) {
+            theta.parameterization.flag <- TRUE
+            theta.parameterization.names <- partable$lhs[ov.var.idx]
+        }
+    }
+
     # shortcut
     REL <- optim.bounds$min.reliability.marker
 
@@ -161,6 +191,13 @@ lav_partable_add_bounds <- function(partable       = NULL,
             }
         }
 
+        # new in 0.6-17: increase observed variances for 'scaled' parameters
+        # if theta parameterization
+        if(theta.parameterization.flag) {
+            sc.idx <- match(theta.parameterization.names, ov.names)
+            OV.VAR[sc.idx] <- OV.VAR[sc.idx]/REL
+        }
+
 
         # we 'process' the parameters per 'type', so we can choose
         # to apply (or not) upper/lower bounds for each type separately
@@ -175,15 +212,26 @@ lav_partable_add_bounds <- function(partable       = NULL,
 
         if(length(par.idx) > 0L) {
             # lower == 0
-            lower.auto[par.idx ] <- 0
+            lower.auto[par.idx] <- 0
 
             # upper == var(ov)
             var.idx <- match(partable$lhs[par.idx], ov.names)
             upper.auto[par.idx] <- OV.VAR[var.idx]
 
+            # if reliability > 0, adapt marker indicators only
+            if(REL > 0) {
+                marker.idx <- which(partable$group == group.values[g] &
+                         partable$op == "~~" &
+                         partable$lhs %in% lv.marker &
+                         partable$lhs == partable$rhs)
+                marker.var.idx <- match(partable$lhs[marker.idx], ov.names)
+
+                # upper = (1-REL)*OVAR
+                upper.auto[marker.idx] <- (1 - REL) * OV.VAR[marker.var.idx]
+            }
+
             # range
-            #bound.range <- upper.auto[par.idx] - lower.auto[par.idx]
-            bound.range <- upper.auto[par.idx] # always assume 0 as lower bound
+            bound.range <- upper.auto[par.idx] - pmax(lower.auto[par.idx], 0)
 
             # enlarge lower?
             if("ov.var" %in% optim.bounds$lower) {
@@ -205,6 +253,12 @@ lav_partable_add_bounds <- function(partable       = NULL,
                 }
             }
 
+            # min.var.ov?
+            min.idx <- which(lower.auto[par.idx] < optim.bounds$min.var.ov)
+            if(length(min.idx) > 0L) {
+                lower.auto[par.idx[min.idx]] <- optim.bounds$min.var.ov
+            }
+
             # requested?
             if("ov.var" %in% optim.bounds$lower) {
                 partable$lower[par.idx] <- lower.auto[par.idx]
@@ -224,7 +278,7 @@ lav_partable_add_bounds <- function(partable       = NULL,
 
         if(lavoptions$std.lv) {
             LV.VAR.LB <- rep(1.0, length(lv.names))
-            LV.VAR.UP <- rep(1.0, length(lv.names))
+            LV.VAR.UB <- rep(1.0, length(lv.names))
         } else {
             for(i in seq_len(length(lv.names))) {
 
@@ -236,7 +290,13 @@ lav_partable_add_bounds <- function(partable       = NULL,
                     marker.var <- OV.VAR[ match(this.lv.marker, ov.names) ]
                     LOWER <- marker.var - (1 - REL)*marker.var
                     LV.VAR.LB[i] <- max(LOWER, optim.bounds$min.var.lv.exo)
+                    #LV.VAR.UB[i] <- marker.var - REL*marker.var
                     LV.VAR.UB[i] <- marker.var
+
+                    # new in 0.6-17
+                    if(theta.parameterization.flag) {
+                        LV.VAR.LB[i] <- REL
+                    }
                 } else {
                     LV.VAR.LB[i] <- optim.bounds$min.var.lv.exo
                     LV.VAR.UB[i] <- max(OV.VAR)
@@ -257,6 +317,13 @@ lav_partable_add_bounds <- function(partable       = NULL,
             endo.idx <- which(lv.names %in% lv.names.endo)
             if(length(endo.idx) > 0L) {
                 LV.VAR.LB2[endo.idx] <- optim.bounds$min.var.lv.endo
+                if(optim.bounds$max.r2.lv.endo != 1) {
+                    LV.VAR.LB2[endo.idx] <- (1 - optim.bounds$max.r2.lv.endo) * LV.VAR.UB[endo.idx]
+                }
+            }
+            exo.idx <- which(!lv.names %in% lv.names.endo)
+            if(length(exo.idx) > 0L && optim.bounds$min.var.lv.exo != 0) {
+                LV.VAR.LB2[exo.idx] <- optim.bounds$min.var.lv.exo
             }
 
             lower.auto[par.idx] <- LV.VAR.LB2[ match(partable$lhs[par.idx],
@@ -265,8 +332,7 @@ lav_partable_add_bounds <- function(partable       = NULL,
                                                      lv.names) ]
 
             # range
-            #bound.range <- upper.auto[par.idx] - lower.auto[par.idx]
-            bound.range <- upper.auto[par.idx] # always assume 0 as lower bound
+            bound.range <- upper.auto[par.idx] - pmax(lower.auto[par.idx], 0)
 
             # enlarge lower?
             if("lv.var" %in% optim.bounds$lower) {
@@ -323,6 +389,12 @@ lav_partable_add_bounds <- function(partable       = NULL,
             lower.auto[par.idx] <- -1 * sqrt(var.all/tmp) # -Inf if tmp==0
             upper.auto[par.idx] <- +1 * sqrt(var.all/tmp) # +Inf if tmp==0
 
+            # if std.lv = TRUE, force 'first' loading to be positive?
+            #if(lavoptions$std.lv) {
+            #    # get index 'first' indicators
+            #    first.idx <- which(!duplicated(partable$lhs[par.idx]))
+            #    lower.auto[par.idx][first.idx] <- 0
+            #}
 
             # range
             bound.range <- upper.auto[par.idx] - lower.auto[par.idx]
@@ -355,6 +427,8 @@ lav_partable_add_bounds <- function(partable       = NULL,
                 }
             }
 
+
+
             # requested?
             if("loadings" %in% optim.bounds$lower) {
                 partable$lower[par.idx] <- lower.auto[par.idx]
@@ -375,15 +449,17 @@ lav_partable_add_bounds <- function(partable       = NULL,
                          partable$op == "~~" &
                          partable$lhs != partable$rhs)
 
-
         if(length(par.idx) > 0L) {
 
             for(i in seq_len(length(par.idx))) {
-
                 # this lhs/rhs
                 this.lhs <- partable$lhs[ par.idx[i] ]
                 this.rhs <- partable$rhs[ par.idx[i] ]
 
+                # 2 possibilities:
+                # - variances are free parameters
+                # - variances are fixed (eg std.lv = TRUE)
+
                 # var idx
                 lhs.var.idx <- which(partable$group == group.values[g] &
                                      partable$op == "~~" &
@@ -403,46 +479,49 @@ lav_partable_add_bounds <- function(partable       = NULL,
                     upper.auto[par.idx[i]] <- +1 * upper.cov
                     lower.auto[par.idx[i]] <- -1 * upper.cov
                 }
+            }
 
-                # enlarge lower?
-                if("covariances" %in% optim.bounds$lower) {
-                    factor <-
-                        lower.factor[ which(optim.bounds$lower=="covariances") ]
-                    if( is.finite(factor) && factor != 1.0 ) {
-                        new.range <- bound.range * factor
-                        ok.idx <- is.finite(new.range)
-                        if(length(ok.idx) > 0L) {
-                            diff <- abs(new.range[ok.idx] - bound.range[ok.idx])
-                            lower.auto[par.idx][ok.idx] <-
-                            lower.auto[par.idx][ok.idx] - diff
-                        }
+            # range
+            bound.range <- upper.auto[par.idx] - lower.auto[par.idx]
+
+            # enlarge lower?
+            if("covariances" %in% optim.bounds$lower) {
+                factor <-
+                    lower.factor[ which(optim.bounds$lower=="covariances") ]
+                if( is.finite(factor) && factor != 1.0 ) {
+                    new.range <- bound.range * factor
+                    ok.idx <- is.finite(new.range)
+                    if(length(ok.idx) > 0L) {
+                        diff <- new.range[ok.idx] - bound.range[ok.idx]
+                        lower.auto[par.idx][ok.idx] <-
+                        lower.auto[par.idx][ok.idx] - diff
                     }
                 }
+            }
 
-                # enlarge upper?
-                if("covariances" %in% optim.bounds$upper) {
-                    factor <-
-                        upper.factor[ which(optim.bounds$upper=="covariances") ]
-                    if( is.finite(factor) && factor != 1.0 ) {
-                        new.range <- bound.range * factor
-                        ok.idx <- is.finite(new.range)
-                        if(length(ok.idx) > 0L) {
-                            diff <- abs(new.range[ok.idx] - bound.range[ok.idx])
-                            upper.auto[par.idx][ok.idx] <-
-                            upper.auto[par.idx][ok.idx] + diff
-                        }
+            # enlarge upper?
+            if("covariances" %in% optim.bounds$upper) {
+                factor <-
+                    upper.factor[ which(optim.bounds$upper=="covariances") ]
+                if( is.finite(factor) && factor != 1.0 ) {
+                    new.range <- bound.range * factor
+                    ok.idx <- is.finite(new.range)
+                    if(length(ok.idx) > 0L) {
+                        diff <- new.range[ok.idx] - bound.range[ok.idx]
+                        upper.auto[par.idx][ok.idx] <-
+                        upper.auto[par.idx][ok.idx] + diff
                     }
                 }
+            }
 
-                # requested?
-                if("covariances" %in% optim.bounds$lower) {
-                    partable$lower[par.idx[i]] <- lower.auto[par.idx[i]]
-                }
-                if("covariances" %in% optim.bounds$upper) {
-                    partable$upper[par.idx[i]] <- upper.auto[par.idx[i]]
-                }
+            # requested?
+            if("covariances" %in% optim.bounds$lower) {
+                partable$lower[par.idx] <- lower.auto[par.idx]
+            }
+            if("covariances" %in% optim.bounds$upper) {
+                partable$upper[par.idx] <- upper.auto[par.idx]
+            }
 
-            } # i
         } # covariances
 
 
