@@ -10,6 +10,7 @@ lav_partable_flat <- function(FLAT = NULL,
                               orthogonal.x     = FALSE,
                               orthogonal.efa   = FALSE,
                               std.lv           = FALSE,
+                              correlation      = FALSE,
                               conditional.x    = FALSE,
                               fixed.x          = TRUE,
                               parameterization = "delta",
@@ -40,6 +41,9 @@ lav_partable_flat <- function(FLAT = NULL,
     lv.names.f   <- lav_partable_vnames(FLAT, type="lv.formative") # formative latent variables
     ov.names     <- lav_partable_vnames(FLAT, type="ov")     # observed variables
     ov.names.x   <- lav_partable_vnames(FLAT, type="ov.x")   # exogenous x covariates
+    lv.names.int <- lav_partable_vnames(FLAT, type="lv.interaction") # lv interactions
+
+
     if(is.null(ov.names.x.block)) {
         ov.names.x.block <- ov.names.x
     }
@@ -274,6 +278,13 @@ lav_partable_flat <- function(FLAT = NULL,
          op <- c(op,  rep("~*~", length(ov.names.ord)))
     }
 
+    # same for correlation structures, but now for ALL variables
+    if(!categorical && correlation) {
+        lhs <- c(lhs, ov.names)
+        rhs <- c(rhs, ov.names)
+         op <- c(op,  rep("~*~", length(ov.names)))
+    }
+
     # 3. INTERCEPTS
     if(meanstructure) {
         #if(conditional.x) {
@@ -327,6 +338,15 @@ lav_partable_flat <- function(FLAT = NULL,
     rhs     <- FLAT$rhs
     mod.idx <- FLAT$mod.idx
 
+    # remove any 'da' entries here (we put them back at the end later)
+    if(any(op == "da")) {
+        da.idx <- which(op == "da")
+        lhs     <- lhs[-da.idx]
+         op     <-  op[-da.idx]
+        rhs     <- rhs[-da.idx]
+        mod.idx <- mod.idx[-da.idx]
+    }
+
     lv.names     <- lav_partable_vnames(FLAT, type="lv")     # latent variables
     ov.names     <- lav_partable_vnames(FLAT, type="ov")     # observed variables
     USER <- data.frame(lhs=lhs, op=op, rhs=rhs, mod.idx=mod.idx,
@@ -529,6 +549,14 @@ lav_partable_flat <- function(FLAT = NULL,
             ustart[lv.rv.idx] <- as.numeric(NA)
               free[lv.rv.idx] <- 1L
         }
+        if(length(lv.names.int) > 0L) {
+            lv.int.idx <- which(op == "~1" &
+                               lhs %in% lv.names.int &
+                               user == 0L)
+            ustart[lv.int.idx] <- as.numeric(NA)
+              free[lv.int.idx] <- 1L
+
+        }
     }
 
     # 4b. fixed effect (only if we have random slopes)
@@ -619,6 +647,21 @@ lav_partable_flat <- function(FLAT = NULL,
           free[delta.idx] <- 0L
     }
 
+    # correlation structure (new in 0.6-13)
+    if(correlation) {
+        var.idx <- which(lhs %in% ov.names &
+                         op == "~~" &
+                         user == 0L &
+                         lhs == rhs)
+        ustart[var.idx] <- 1L
+          free[var.idx] <- 0L
+
+        delta.idx <- which(op == "~*~" &
+                           user == 0L)
+        ustart[delta.idx] <- 1.0
+          free[delta.idx] <- 0L
+    }
+
     # group proportions (group 1L)
     if(group.w.free) {
         group.idx <- which(lhs == "group" & op == "%")
