@@ -52,6 +52,7 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L, shift = 0L,
     # in package nlme
     x <- as.matrix(x) # just in case
     y <- x; y <- unclass(y)
+    attributes(y)[c("header","footer")] <- NULL
     ll <- lower.tri(x, diag = TRUE)
     y[ ll] <- format(round(x[ll], digits = nd))
     y[!ll] <- ""
@@ -74,7 +75,17 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L, shift = 0L,
             rownames(y) <- empty.string
         }
     }
+
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     print(y, ..., quote = FALSE, right = TRUE)
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
@@ -82,6 +93,7 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L, shift = 0L,
 print.lavaan.matrix <- function(x, ..., nd = 3L, shift = 0L) {
     x <- as.matrix(x) # just in case
     y <- unclass(x)
+    attributes(y)[c("header","footer")] <- NULL
     if(!is.null(colnames(x))) {
       colnames(y) <- abbreviate(colnames(x), minlength = nd + 3L)
     }
@@ -93,15 +105,29 @@ print.lavaan.matrix <- function(x, ..., nd = 3L, shift = 0L) {
             rownames(y) <- empty.string
         }
     }
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     print( round(y, nd), right = TRUE, ... )
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
 print.lavaan.vector <- function(x, ..., nd = 3L, shift = 0L) {
     y <- unclass(x)
+    attributes(y)[c("header","footer")] <- NULL
     #if(!is.null(names(x))) {
     #    names(y) <- abbreviate(names(x), minlength = nd + 3)
     #}
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     if(shift > 0L) {
         empty.string <- strrep(x = " ", times = shift)
         tmp <- format(y, digits = nd, width = 2L + nd)
@@ -110,10 +136,15 @@ print.lavaan.vector <- function(x, ..., nd = 3L, shift = 0L) {
     } else {
         print( round(y, nd), right = TRUE, ... )
     }
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
-print.lavaan.character <- function(x) {
+print.lavaan.character <- function(x, ...) {
     cat(x)
     invisible(x)
 }
@@ -148,6 +179,32 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     if(header) {
         cat("\nParameter Estimates:\n\n")
 
+        # info about parameterization (if categorical only)
+        categorical.flag <- attr(x, "categorical")
+        if(categorical.flag) {
+            # container
+            c1 <- c2 <- character(0L)
+
+            # which parameterization?
+            c1 <- c(c1, "Parameterization")
+            tmp.txt <- attr(x, "parameterization")
+            c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                                      substring(tmp.txt, 2), sep = ""))
+
+            # format c1/c2
+            c1 <- format(c1, width = 38L)
+            c2 <- format(c2,
+                   width = 13L + max(0, (nd - 3L)) * 4L, justify = "right")
+
+            # create character matrix
+            M <- cbind(c1, c2, deparse.level = 0)
+            colnames(M) <- rep("",  ncol(M))
+            rownames(M) <- rep(" ", nrow(M))
+
+            # print
+            write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+        }
+
         # info about standard errors (if we have x$se only)
         # 1. information
         # 2. se
@@ -810,8 +867,6 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
     if(struc) {
         cat("Structural Part\n")
         cat("\n")
-        #print.lavaan.parameterEstimates(y$PE, ..., nd = nd)
-
         print(summary(y$STRUC.FIT, fit.measures = FALSE, estimates = FALSE,
                                     modindices = FALSE))
         FIT <- fitMeasures(y$STRUC.FIT, fit.measures="default")
@@ -822,7 +877,7 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
     PE <- parameterEstimates(y$STRUC.FIT, ci = FALSE,
                              remove.eq = FALSE, remove.system.eq = TRUE,
                              remove.ineq = FALSE, remove.def = FALSE,
-                             remove.nonfree = FALSE,
+                             remove.nonfree = FALSE, remove.unused = TRUE,
                              output = "text", header = TRUE)
     print.lavaan.parameterEstimates(PE, ..., nd = nd)
 
@@ -1147,7 +1202,10 @@ print.lavaan.summary <- function(x, ..., nd = 3L) {
         sam.mm.rel    <- y$sam$sam.mm.rel
         sam.struc.fit <- y$sam$sam.struc.fit
         ngroups       <- y$sam$ngroups
+        nlevels       <- y$sam$nlevels
         group.label   <- y$sam$group.label
+        level.label   <- y$sam$level.label
+        block.label   <- y$sam$block.label
 
         # measurement
         tmp <- sam.mm.table
@@ -1161,13 +1219,13 @@ print.lavaan.summary <- function(x, ..., nd = 3L) {
         if(sam.method == "local") {
             # reliability information
             c1 <- c2 <- character(0L)
-            if(ngroups == 1L) {
+            if(ngroups == 1L && nlevels == 1L) {
                 cat("\n")
                 cat("  Model-based reliability latent variables:\n\n")
                 tmp <- data.frame(as.list(sam.mm.rel[[1]]))
                 class(tmp) <- c("lavaan.data.frame", "data.frame")
                 print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
-            } else {
+            } else if(ngroups > 1L && nlevels == 1L) {
                 cat("\n")
                 cat("  Model-based reliability latent variables (per group):\n")
                 for(g in 1:ngroups) {
@@ -1178,6 +1236,28 @@ print.lavaan.summary <- function(x, ..., nd = 3L) {
                     class(tmp) <- c("lavaan.data.frame", "data.frame")
                     print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
                 }
+            } else if(ngroups == 1L && nlevels > 1L) {
+                cat("\n")
+                cat("  Model-based reliability latent variables (per level):\n")
+                for(g in 1:nlevels) {
+                    cat("\n")
+                    cat("  Level ", g, " [", level.label[g], "]:\n\n",
+                        sep = "")
+                    tmp <- data.frame(as.list(sam.mm.rel[[g]]))
+                    class(tmp) <- c("lavaan.data.frame", "data.frame")
+                    print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+                }
+            } else if(ngroups > 1L && nlevels > 1L) {
+                cat("\n")
+                cat("  Model-based reliability latent variables (per group/level):\n")
+                for(g in 1:length(block.label)) {
+                    cat("\n")
+                    cat("  Group/Level ", g, " [", block.label[g], "]:\n\n",
+                        sep = "")
+                    tmp <- data.frame(as.list(sam.mm.rel[[g]]))
+                    class(tmp) <- c("lavaan.data.frame", "data.frame")
+                    print(tmp, row.names = rep(" ", nrow(tmp)), nd = nd)
+                }
             }
 
             cat("\n")
