@@ -73,6 +73,13 @@ lav_samplestats_from_data <- function(lavdata           = NULL,
         }
     }
 
+    # sample.cov.robust cannot be used if sampling weights are used
+    if(lavoptions$sample.cov.robust) {
+        if(!is.null(WT[[1]])) {
+            stop("lavaan ERROR: sample.cov.robust = TRUE does not work (yet) if sampling weights are provided.")
+        }
+    }
+
     # sample statistics per group
 
     # joint (y,x)
@@ -191,13 +198,15 @@ lav_samplestats_from_data <- function(lavdata           = NULL,
     for(g in 1:ngroups) {
 
         # check nobs
-        if(nobs[[g]] < 2L) {
-            if(nobs[[g]] == 0L) {
-                stop("lavaan ERROR: data contains no observations",
+        if(is.null(WT[[g]])) {
+            if(nobs[[g]] < 2L) {
+                if(nobs[[g]] == 0L) {
+                    stop("lavaan ERROR: data contains no observations",
                      ifelse(ngroups > 1L, paste(" in group ", g, sep=""), ""))
-            } else {
-                stop("lavaan ERROR: data contains only a single observation",
+                } else {
+                    stop("lavaan ERROR: data contains only a single observation",
                      ifelse(ngroups > 1L, paste(" in group ", g, sep=""), ""))
+                }
             }
         }
 
@@ -580,6 +589,13 @@ lav_samplestats_from_data <- function(lavdata           = NULL,
                     }
                     var[[g]]  <- diag(cov[[g]])
                     mean[[g]] <- out$center
+                } else if(lavoptions$sample.cov.robust) {
+                    # fixme: allow prob/max.it to be options
+                    out <- lav_cov_huber(Y = X[[g]], prob = 0.95,
+                                         max.it = 200L, tol = 1e-07)
+                    cov[[g]]  <- out$Sigma
+                    var[[g]]  <- diag(cov[[g]])
+                    mean[[g]] <- out$Mu
                 } else {
                     cov[[g]]  <-   stats::cov(X[[g]], use = "pairwise")
                     # rescale cov by (N-1)/N? (only COV!)
@@ -1678,7 +1694,16 @@ lav_samplestats_cluster_patterns <- function(Y = NULL, Lp = NULL,
         Y2c <- t( t(Y2) - Y1.means )
 
         # compute S.w
-        Y1a <- Y1 - Y2[cluster.idx, , drop = FALSE]
+        # center within variables by grand mean instead of group mean?
+        # (YR: apparently not for S.PW)
+
+        Y2a <- Y2
+        #if(length(within.idx) > 0L) {
+        #    for(i in 1:length(within.idx)) {
+        #        Y2a[, within.idx[i]] <- Y1.means[within.idx[i]]
+        #    }
+        #}
+        Y1a <- Y1 - Y2a[cluster.idx, , drop = FALSE]
         S.w <- lav_matrix_crossprod(Y1a) / (N - nclusters)
 
         # S.b
